package com.ekexiu.portal.dao;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.Column;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.Or;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.Where;
import org.jfw.apt.orm.annotation.dao.method.operator.DeleteWith;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.LimitSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.PageSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.QueryVal;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.Alias;
import org.jfw.apt.orm.annotation.dao.param.LessThan;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.apt.orm.core.defaultImpl.IntHandler;
import org.jfw.util.PageQueryResult;

import com.ekexiu.portal.po.User;

@DAO
public interface UserDao {

	@Nullable
	@SelectOne
	User query(Connection con,String id)throws SQLException;
	
	@Or
	@Nullable
	@SelectOne
	User queryByEmailOrMobilePhone(Connection con,@Alias({"email","mobilePhone"}) String key)throws SQLException;
	
	@SelectList
	List<User> queryList(Connection con)throws SQLException;
	
	@LimitSelect
	@OrderBy("ORDER BY CREATE_TIME DESC")
	List<User> queryLimit(Connection con,String inviterId,@LessThan String createTime,int rows)throws SQLException;
	
	@SelectList
	List<User> queryByInviter(Connection con,String inviterId)throws SQLException;
	
	@QueryVal
	@From(User.class)
	@Column(handlerClass=IntHandler.class,value={"COUNT(1)"})
	@DefaultValue("0")
	int queryInviter(Connection con,String inviterId)throws SQLException;
	
	@DeleteWith
	@From(User.class)
	int delete(Connection con, String id) throws SQLException;
	
	@Insert
	int insert(Connection con,User user)throws SQLException;
	
	@UpdateWith
	@From(User.class)
	int updateActiveTime(Connection con,@Set String activeTime,String id)throws SQLException;
	
	@UpdateWith
	@From(User.class)
	int updateSendMailStatus(Connection con,@Set Integer sendMailStatus,String id)throws SQLException;
	
	@UpdateWith
	@From(User.class)
	int updateInviteCode(Connection con,@Set String inviteCode,String id)throws SQLException;
	
	@UpdateWith
	@From(User.class)
	@Where("ACTIVE_TIME IS NULL AND PASSWD = '11111111111111111111111111111111'")
	int updatePasswdAndActive(Connection con,@Set String passwd,@Set String activeTime,String id) throws SQLException;
	
	@UpdateWith
	@From(User.class)
	int updatePasswd(Connection con,@Set String passwd,String id,@Alias("passwd")String key) throws SQLException;
	
	@UpdateWith
	@From(User.class)
	int updateEmail(Connection con,@Set String email,String id)throws SQLException;
	@UpdateWith
	@From(User.class)
	int updateMobilePhone(Connection con,@Set String mobilePhone,String id)throws SQLException;
	
	@UpdateWith
	@From(User.class)
	int updatePassword(Connection con,@Set String passwd,String id)throws SQLException;
	
	@UpdateWith
	@From(User.class)
	@Where("ACTIVE_TIME IS NULL")
	int updatePasswdWithMobile(Connection con,@Set String passwd,@Set String activeTime,@Alias("mobilePhone") String key)throws SQLException;
	
	@Or
	@UpdateWith
	@From(User.class)
	int updatePasswordWithMobileOrEmail(Connection con,@Set String passwd,@Alias({"email","mobilePhone"}) String key)throws SQLException;
	
	@QueryVal
	@Column(handlerClass=IntHandler.class, value = { "COUNT(1)" })
	@From(User.class)
	@DefaultValue("0")
	int countAll(Connection con)throws SQLException;
	
	@QueryVal
	@Column(handlerClass=IntHandler.class, value = { "COUNT(1)" })
	@From(User.class)
	@DefaultValue("0")
	@Where("PASSWD<>'11111111111111111111111111111111'")
	int countActived(Connection con)throws SQLException;
	
	/*
	 * 查询所有未激活用户
	 */
	@PageSelect
	@Where("PASSWD='11111111111111111111111111111111'")
	@OrderBy("ORDER BY CREATE_TIME")
	PageQueryResult<User> pageQuery(Connection con,int pageSize,int pageNo)throws SQLException;
	
}
