package com.ekexiu.portal.platform;

import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.Exclude;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.LimitColumn;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.Where;
import org.jfw.apt.orm.annotation.dao.method.operator.Delete;
import org.jfw.apt.orm.annotation.dao.method.operator.DeleteWith;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.LimitQuery;
import org.jfw.apt.orm.annotation.dao.method.operator.PageQuery;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.Update;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.Alias;
import org.jfw.apt.orm.annotation.dao.param.GroupSqlColumn;
import org.jfw.apt.orm.annotation.dao.param.GtEq;
import org.jfw.apt.orm.annotation.dao.param.In;
import org.jfw.apt.orm.annotation.dao.param.Like;
import org.jfw.apt.orm.annotation.dao.param.LtEq;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.apt.orm.annotation.dao.param.SqlColumn;
import org.jfw.apt.orm.core.defaultImpl.LongHandler;
import org.jfw.apt.orm.core.defaultImpl.StringHandler;
import org.jfw.util.PageQueryResult;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

@DAO
public interface PlatformDao {

	@SelectOne
	@Nullable
	@Where("STATE='1'")
	PlatformUser login(Connection con, String email, String passwd) throws SQLException;

	@Nullable
	@SelectOne
	PlatformInfo queryInfo(Connection con, String id) throws SQLException;

	@SelectOne
	@Nullable
	PlatformUser queryUser(Connection con, String email) throws SQLException;

	@SelectOne
	@Nullable
	PlatformUser queryUserById(Connection con, String id) throws SQLException;

	@SelectList
	List<PlatformInfo> queryInfos(Connection con, @In String[] id) throws SQLException;

	@UpdateWith
	@From(PlatformUser.class)
	int updatePasswd(Connection con, String id, @Alias("passwd") String oldpw, @Set String passwd) throws SQLException;

	@UpdateWith
	@From(PlatformUser.class)
	int updatePasswd(Connection con, String id, String email, @Alias("passwd") String oldpw, @Set String passwd) throws SQLException;

	@UpdateWith
	@From(PlatformUser.class)
	int updateMail(Connection con, String id, @Alias("email") String oldemail, String passwd, @Set String email) throws SQLException;

	@Update
	int update(Connection con, PlatformInfo pi) throws SQLException;

	@Insert
	int insert(Connection con, ResidentOrg org) throws SQLException;

	@Insert
	int insert(Connection con, ButtedOrg org) throws SQLException;

	@Insert
	int insert(Connection con, ButtedProfessor professor) throws SQLException;

	@Insert
	int insert(Connection con, ReferencedArticle article) throws SQLException;

	@Delete
	int delete(Connection con, ResidentOrg org) throws SQLException;

	@Delete
	int delete(Connection con, ButtedOrg org) throws SQLException;

	@Delete
	int delete(Connection con, ButtedProfessor professor) throws SQLException;

	@Delete
	int delete(Connection con, ReferencedArticle article) throws SQLException;

	@SelectList
	List<ResidentOrg> queryResidentOrgs(Connection con, String pid) throws SQLException;

	@SelectList
	List<ButtedOrg> queryButtedOrgs(Connection con, String pid) throws SQLException;

	@SelectList
	List<ButtedProfessor> queryButtedProfessors(Connection con, String pid) throws SQLException;

	@SelectList
	List<ReferencedArticle> queryReferencedArticles(Connection con, String pid) throws SQLException;

	@SelectOne
	@Nullable
	ResidentOrg queryResidentOrg(Connection con, String pid, String oid) throws SQLException;

	@SelectOne
	@Nullable
	ButtedOrg queryButtedOrg(Connection con, String pid, String oid) throws SQLException;

	@SelectOne
	@Nullable
	ButtedProfessor queryButtedProfessor(Connection con, String pid, String uid) throws SQLException;

	@SelectOne
	@Nullable
	ReferencedArticle queryReferencedArticle(Connection con, String pid, String aid) throws SQLException;

	@DeleteWith
	@From(ResidentOrg.class)
	int deleteResidentOrg(Connection con, String pid, String oid) throws SQLException;

	@DeleteWith
	@From(ButtedOrg.class)
	int deleteButtedOrg(Connection con, String pid, String oid) throws SQLException;

	@DeleteWith
	@From(ButtedProfessor.class)
	int deleteButtedProfessor(Connection con, String pid, String uid) throws SQLException;

	@DeleteWith
	@From(ReferencedArticle.class)
	int deleteReferencedArticle(Connection con, String pid, String aid) throws SQLException;

	@LimitQuery
	@OrderBy(cols = { @LimitColumn(value = "time", column = "R.CREATE_TIME", handlerClass = StringHandler.class, asc = false),
			@LimitColumn(value = "oid", column = "O.ID", asc = false, handlerClass = StringHandler.class) }, value = "")
	@Exclude({ "descp" })
	List<ResidentOrgInfo> queryResientOrgInfo(Connection con, String time, String oid,
			@SqlColumn(handlerClass = StringHandler.class, value = { "R.PID=?" }) String pid,
			@Nullable @GroupSqlColumn(handlerClass = StringHandler.class, isAnd = false, value = { "O.NAME LIKE ?", "O.FOR_SHORT LIKE ?" }) String key,
			int rows) throws SQLException;
	
	
	@PageQuery
	@OrderBy(value = "ORDER BY R.CREATE_TIME DESC")
	@Exclude({ "descp" })
	PageQueryResult<ResidentOrgInfo> pqResientOrgInfo(Connection con,@SqlColumn(handlerClass = StringHandler.class, value = { "R.PID=?" }) String pid,
			@Nullable @GroupSqlColumn(handlerClass = StringHandler.class, isAnd = false, value = { "O.NAME LIKE ?", "O.FOR_SHORT LIKE ?" }) String key,
			int pageSize,int pageNo) throws SQLException;

	@LimitQuery
	@OrderBy(cols = { @LimitColumn(value = "time", column = "B.CREATE_TIME", handlerClass = StringHandler.class, asc = false),
			@LimitColumn(value = "oid", column = "O.ID", asc = false, handlerClass = StringHandler.class) }, value = "")
	@Exclude({ "descp" })
	List<ButtedOrgInfo> queryButtedOrgInfo(Connection con, String time, String oid,
			@SqlColumn(handlerClass = StringHandler.class, value = { "B.PID=?" }) String pid, int rows) throws SQLException;

	@PageQuery
	@OrderBy(value = "ORDER BY BUTTE_TIME DESC,ID ASC")
	@Exclude({ "descp" })
	PageQueryResult<ButtedOrgInfo> pageQueryButtedOrgInfo(Connection con, @SqlColumn(handlerClass = StringHandler.class, value = { "B.PID=?" }) String pid,
			int pageSize, int pageNo) throws SQLException;

	@LimitQuery
	@OrderBy(cols = { @LimitColumn(value = "time", column = "B.CREATE_TIME", handlerClass = StringHandler.class, asc = false),
			@LimitColumn(value = "uid", column = "P.ID", asc = false, handlerClass = StringHandler.class) }, value = "")
	@Exclude({ "descp" })
	List<ButtedProfessorInfo> queryButtedProfessorInfo(Connection con, String time, String uid,
			@SqlColumn(handlerClass = StringHandler.class, value = { "B.PID=?" }) String pid, int rows) throws SQLException;

	@PageQuery
	@OrderBy(value = "ORDER BY BUTTE_TIME DESC,ID ASC")
	@Exclude({ "descp" })
	PageQueryResult<ButtedProfessorInfo> queryButtedProfessorInfo(Connection con,
			@SqlColumn(handlerClass = StringHandler.class, value = { "B.PID=?" }) String pid, int pageSize, int pageNo) throws SQLException;

	@LimitQuery
	@OrderBy(cols = { @LimitColumn(value = "time", column = "PTIME", handlerClass = StringHandler.class, asc = false),
			@LimitColumn(value = "shareId", handlerClass = LongHandler.class) }, value = "")
	List<PlatformWare> queryWare(Connection con, @GroupSqlColumn(handlerClass = StringHandler.class, value = { "WNAME LIKE ?", "KEYWORDS LIKE ?",
			"FNAME LIKE ?", "SNAME LIKE ?", "CNT LIKE ?"},isAnd = false) @Nullable String key, String pid, String time, long shareId, int rows) throws SQLException;

	@LimitQuery
	@OrderBy(cols = { @LimitColumn(value = "time", column = "PTIME", handlerClass = StringHandler.class, asc = false),
			@LimitColumn(value = "shareId", handlerClass = LongHandler.class) }, value = "")
	List<PlatformResource> queryResource(Connection con, @GroupSqlColumn(handlerClass = StringHandler.class, value = { "RNAME LIKE ?", "KEYWORDS LIKE ?",
			"FNAME LIKE ?", "SNAME LIKE ?", "CNT LIKE ?" ,"COMP LIKE ?"},isAnd = false) @Nullable String key, String pid, String time, long shareId, int rows) throws SQLException;

	/*
	 * 运营系统
	 */
	@Insert
	int insert(Connection con, PlatformInfo platformInfo) throws SQLException;

	@Insert
	int insert(Connection con, PlatformUser platformUser) throws SQLException;

	@UpdateWith
	@From(PlatformUser.class)
	int updateEmail(Connection con, String id, @Set String email) throws SQLException;

	@UpdateWith
	@From(PlatformInfo.class)
	int updateName(Connection con, String id, @Set String name) throws SQLException;

	@Nullable
	@SelectOne
	PlatformInfo queryInfoByName(Connection con, String name) throws SQLException;

	@PageQuery
	@OrderBy(" ORDER BY create_time DESC NULLS LAST")
	PageQueryResult<PlatformInfo> query(Connection con, @Nullable @Like String name, @Nullable @GtEq @Alias("createTime") String bt, @Nullable @Alias("createTime") @LtEq String et, int pageSize, int pageNo) throws SQLException;

	@PageQuery
	@OrderBy(value = "ORDER BY BUTTE_TIME DESC,ID ASC")
	@Exclude({"descp"})
	PageQueryResult<ButtedProfessorInfo> queryButtedProfessorInfo(Connection con,
																  @SqlColumn(handlerClass = StringHandler.class, value = { "B.PID=?" }) String pid,@Nullable @SqlColumn(handlerClass = StringHandler.class, value = { "P.NAME LIKE ?" }) String name, int pageSize, int pageNo) throws SQLException;

	@PageQuery
	@OrderBy(value = "ORDER BY BUTTE_TIME DESC,ID ASC")
	@Exclude({"descp"})
	PageQueryResult<ButtedOrgInfo> queryButtedOrgInfo(Connection con,
																  @SqlColumn(handlerClass = StringHandler.class, value = { "B.PID=?" }) String pid,@Nullable @SqlColumn(handlerClass = StringHandler.class, value = { "O.NAME LIKE ?" }) String name, int pageSize, int pageNo) throws SQLException;
}
