package com.ekexiu.portal.product;

import com.ekexiu.portal.dao.WatchDao;
import com.ekexiu.portal.service.KeyWordService;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.RequestParam;
import org.jfw.apt.web.annotation.param.Upload;
import org.jfw.util.JpgUtil;
import org.jfw.util.ListUtil;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.io.IoUtil;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;
import org.jfw.util.jdbc.ResultSetExtractor;
import org.jfw.util.web.fileupload.Item;
import org.jfw.util.web.fileupload.UploadItemIterator;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by TT on 2018/7/9.
 */
@Path("/product")
public class ProductService {
    private static final AtomicInteger FN_IDX = new AtomicInteger(1);

    private File imgPath;

    private int imgMaxWidth = 70;

    @Autowrie
    private ProductDao productDao;
    @Autowrie
    private WatchDao watchDao;

    @Autowrie
    private KeyWordService keyWordService;

    public ProductDao getProductDao() {
        return productDao;
    }

    public void setProductDao(ProductDao productDao) {
        this.productDao = productDao;
    }

    public WatchDao getWatchDao() {
        return watchDao;
    }

    public void setWatchDao(WatchDao watchDao) {
        this.watchDao = watchDao;
    }

    public KeyWordService getKeyWordService() {
        return keyWordService;
    }

    public void setKeyWordService(KeyWordService keyWordService) {
        this.keyWordService = keyWordService;
    }

    public File getImgPath() {
        return imgPath;
    }

    public void setImgPath(File imgPath) {
        this.imgPath = imgPath;
    }

    public int getImgMaxWidth() {
        return imgMaxWidth;
    }

    public void setImgMaxWidth(int imgMaxWidth) {
        this.imgMaxWidth = imgMaxWidth;
    }

    /**
     * 上传文件
     *
     * @param it
     * @return
     * @throws Exception
     */
    @Path("/upload")
    @Post
    public List<UploadFile> upload(@Upload UploadItemIterator it) throws Exception {
        List<UploadFile> ret = new LinkedList<>();
        try {
            while (it.hasNext()) {
                Item item = it.next();
                if (!item.isFormField()) {
                    String name = normalizeFileName(item.getName());
                    int index = name.lastIndexOf('.');
                    String ext = index >= 0 ? name.substring(index + 1) : "";
                    ext = ext.trim();
                    long fileLen = 0;
                    int len = 0;
                    UploadFile uf = this.buildTargetFile(ext);
                    uf.setName(name);
                    File file = uf.getFn();
                    ret.add(uf);
                    byte[] buf = new byte[8092];
                    OutputStream of = new FileOutputStream(file);
                    try {
                        InputStream in = item.getInputStream();
                        try {
                            while ((len = in.read(buf)) >= 0) {
                                if (len > 0) {
                                    of.write(buf, 0, len);
                                    fileLen += len;
                                }
                            }
                            uf.setSize(fileLen);
                        } finally {
                            in.close();
                        }
                    } finally {
                        of.close();
                    }
                }
            }
            return ret;
        } catch (Exception e) {
            this.remove(ret);
            throw e;
        } finally {
            if (it != null)
                it.clean();
        }
    }

    public String insert(Connection con, Product product, String[] professor) throws SQLException, IOException {
        String id = StringUtil.buildUUID();
        product.setId(id);
        this.productDao.insert(con, product);
        this.saveSmallImg(product.getImages());
        String[] kws = null;
        if (product.getKeywords() != null) {
            kws = ListUtil.splitTrimExcludeEmpty(product.getKeywords(), ',').toArray(new String[0]);
        }
        if (product.getState().equals("1")) {
            keyWordService.refreshProduct(con, id, kws);
        }
        if (professor != null && professor.length > 0) {
            for (String pid : professor) {
                ProductPro productPro = new ProductPro();
                productPro.setId(id);
                productPro.setProfessor(pid);
                productDao.insert(con, productPro);
            }
        }
        return id;
    }

    public boolean update(Connection con, Product product, String[] professor) throws SQLException, IOException {
        String id = product.getId();
        if (productDao.update(con, product) > 0) {
            this.saveSmallImg(product.getImages());
            String[] kws = null;
            if (product.getKeywords() != null) {
                kws = ListUtil.splitTrimExcludeEmpty(product.getKeywords(), ',').toArray(new String[0]);
            }
            if (product.getState().equals("1")) {
                keyWordService.refreshProduct(con, id, kws);
            } else {
                keyWordService.deleteProduct(con, id);
            }
            productDao.deleteProductPro(con, id);
            if (professor != null && professor.length > 0) {
                for (String pid : professor) {
                    ProductPro pp = new ProductPro();
                    pp.setId(id);
                    pp.setProfessor(pid);
                    productDao.insert(con, pp);
                }
            }
            return true;
        }
        return false;
    }

    /**
     * 新增发布状态的产品
     *
     * @param con
     * @param product
     * @param professor
     * @return
     * @throws SQLException
     * @throws IOException
     */
    @Post
    @Path("/publish")
    public String publish(@JdbcConn(true) Connection con, @RequestParam(excludeFields = {"id", "state", "createTime", "modifytime", "shareid", "pageViews", "sortFirst"}) Product product, @Nullable String[] professor) throws SQLException, IOException {
        product.setState("1");
        return insert(con, product, professor);
    }

    /**
     * 新增草稿状态的产品
     *
     * @param con
     * @param product
     * @param professor
     * @return
     * @throws SQLException
     * @throws IOException
     */
    @Post
    @Path("/draft")
    public String draft(@JdbcConn(true) Connection con, @RequestParam(excludeFields = {"id", "state", "createTime", "modifyTime", "shareId", "pageViews", "sortFirst"}) Product product, @Nullable String[] professor) throws SQLException, IOException {
        product.setState("2");
        return insert(con, product, professor);
    }

    /**
     * 修改为草稿状态产品
     *
     * @param con
     * @param product
     * @return
     * @throws SQLException
     * @throws IOException
     */
    @Post
    @Path("/draft/update")
    public boolean updateDraft(@JdbcConn(true) Connection con, @RequestParam(excludeFields = {"id", "state", "createTime", "modifyTime", "shareId", "pageViews", "sortFirst"}) Product product) throws SQLException, IOException {
        product.setState("2");
        return update(con, product, null);
    }

    /**
     * 修改为发布状态产品
     *
     * @param con
     * @param product
     * @param professor
     * @return
     * @throws SQLException
     * @throws IOException
     */
    @Post
    @Path("/publish/update")
    public boolean updatePublish(@JdbcConn(true) Connection con, @RequestParam(excludeFields = {"id", "state", "createTime", "modifyTime", "shareId", "pageViews", "sortFirst"}) Product product, @Nullable String[] professor) throws SQLException, IOException {
        product.setState("1");
        return update(con, product, professor);
    }

    /**
     * 查询指定的产品(单个)
     *
     * @param con
     * @param id
     * @param shareId
     * @return
     * @throws SQLException
     */
    @Get
    @Path("/qo")
    public Product queryOne(@JdbcConn Connection con, @Nullable String id, @DefaultValue("-1") long shareId) throws SQLException {
        if (id != null) {
            return productDao.queryById(con, id);
        } else {
            return productDao.queryByShareId(con, shareId);
        }
    }

    /**
     * 查询指定的服务（多个）
     *
     * @param con
     * @param id
     * @return
     * @throws SQLException
     */
    @Get
    @Path("/qm")
    public List<Product> queryByIds(@JdbcConn Connection con, String[] id) throws SQLException {
        return productDao.queryById(con, id);
    }

    /**
     * 查询产品联系人
     *
     * @param con
     * @param id
     * @return
     * @throws SQLException
     */
    @Get
    @Path("/pro")
    public List<ProductPro> queryPro(@JdbcConn Connection con, String id) throws SQLException {
        return productDao.queryPro(con, id);
    }

    /**
     * 根据产品名称、内容、关键词查询指定企业的产品（草稿在前，按最后修改时间由新到旧排序；已发布产品在后，其中未设置联系人的在前，按发布时间由新到旧排序）
     * @param con
     * @param key
     * @param oid
     * @param pageSize
     * @param pageNo
     * @return
     * @throws SQLException
     */
    @Get
    @Path("/pq/org/search")
    public PageQueryResult<CountProduct> pageQueryOrg(@JdbcConn Connection con, @Nullable String key, String oid, int pageSize, int pageNo) throws SQLException {
        if (key != null) {
            key = "%" + key + "%";
        }
        final List<CountProduct> list = productDao.queryByOwner(con, key, oid);
        final PageQueryResult<CountProduct> result = new PageQueryResult<>();
        if (list != null && list.size() > 0) {
            StringBuilder sb = new StringBuilder();
            sb.append("SELECT ID,COUNT(1) FROM PRODUCT_PRO WHERE ID IN (");
            for (int i = 0; i < list.size(); ++i) {
                sb.append("?,");
            }
            sb.setCharAt(sb.length() - 1, ')');
            sb.append(" GROUP BY ID");
            JdbcUtil.queryList(con, sb.toString(), new ResultSetExtractor<CountProduct>() {
                @Override
                public CountProduct extractData(ResultSet resultSet) throws SQLException {
                    String id = resultSet.getString(1);
                    long c = resultSet.getLong(2);
                    for (CountProduct cp : list) {
                        if (cp.getId().equals(id)) {
                            cp.setNum(c);
                            return cp;
                        }
                    }
                    return null;
                }
            }, new PreparedStatementConfig() {
                @Override
                public void config(PreparedStatement preparedStatement) throws SQLException {
                    int i = 1;
                    for (CountProduct cp : list) {
                        preparedStatement.setString(i++, cp.getId());
                    }
                }
            });
            Collections.sort(list, COUNT_PRODUCT_COMPARATOR);
            result.setTotal(list.size());
            if (pageNo == 1) {
                result.setPageNo(1);
                if (pageSize >= pageNo) {
                    result.setData(list);
                } else {
                    result.setData(list.subList(0, pageSize));
                }
            } else if (pageNo > 1) {
                int index = list.size() / pageSize;
                if (list.size() % pageSize != 0) {
                    ++index;
                }
                if (pageNo > index) {
                    pageNo = index;
                }
                result.setPageNo(pageNo);
                --pageNo;
                int begin = pageNo * pageSize - 1;
                int end = begin + pageSize;
                if (end > list.size()) {
                    end = list.size();
                }
                result.setData(list.subList(begin, end));
            }
        } else {
            result.setTotal(0);
        }
        return result;
    }

    /**
     * 逻辑删除产品
     *
     * @param con
     * @param id
     * @return
     * @throws SQLException
     */
    @Post
    @Path("/delete")
    public int delete(@JdbcConn(true)Connection con,String id)throws SQLException {
        watchDao.deleteWatch(con,(short)10, id);
        return productDao.logicDelete(con, id);
    }

    /**
     * 制定所有者下发布状态下的产品数量
     * @param con
     * @param owner
     * @return
     * @throws SQLException
     */
    @Get
    @Path("/count/publish")
    public long countPublish(@JdbcConn Connection con,String owner)throws SQLException {
        return productDao.countPublish(con, owner);
    }

    /**
     * 指定所有者下发布状态下的产品（下拉加载，modifyTime）
     * @param con
     * @param owner
     * @param modifyTime
     * @param rows
     * @return
     * @throws SQLException
     */
    @Get
    @Path("/publish")
    public List<Product> queryPublish(@JdbcConn Connection con,String owner,@DefaultValue("\"9\"")String modifyTime,int rows)throws SQLException {
        return productDao.queryPublish(con, owner, modifyTime, rows);
    }

    /**
     * 增加浏览量
     * @param con
     * @param id
     * @return
     * @throws SQLException
     */
    @Post
    @Path("/incPageViews")
    public int incPageViews(@JdbcConn(true)Connection con,String id)throws SQLException {
        return productDao.incPageViews(con, id);
    }

    /**
     * 搜索产品（下拉加载，sortFirst，time,id）
     *
     * @param con
     * @param key
     * @param sortFirst
     * @param time
     * @param id
     * @param rows
     * @return
     * @throws SQLException
     */
    @Get
    @Path("/index/search")
    public List<Product> indexSearch(@JdbcConn Connection con,@Nullable String key,@DefaultValue("Long.MAX_VALUE")long sortFirst,@DefaultValue("\"9\"")String time,@DefaultValue("\"Z\"")String id,int rows)throws SQLException {
        if (key != null) {
            key = "%" + key + "%";
        }
        return productDao.indexSearch(con, key, sortFirst, time, id, rows);
    }

    /**
     * 指定发布人发布中的产品 浏览量倒序
     * @param con
     * @param owner
     * @param id
     * @param rows
     * @return
     * @throws SQLException
     */
    @Get
    @Path("/byOwnerWithPageViews")
    public List<Product> queryBySameKwWithPageViews(@JdbcConn Connection con, String owner, @Nullable String id, int rows)throws SQLException {
        return productDao.queryByOwnerWithPageViews(con, owner, id, rows);
    }




    private static final Comparator<CountProduct> COUNT_PRODUCT_COMPARATOR = new Comparator<CountProduct>() {
        @Override
        public int compare(CountProduct o1, CountProduct o2) {
            int v = o1.getState().compareTo(o2.getState());
            if (v == 0) {
                long l1 = o1.getNum();
                long l2 = o2.getNum();
                if (l1 == 0 && l2 != 0) {
                    return 1;
                } else if (l2 == 0 && l1 != 0) {
                    return -1;
                } else {
                    return o1.getModifyTime().compareTo(o2.getModifyTime());
                }
            }
            return v;
        }
    };


    private void saveSmallImg(String imgs) throws IOException {
        if (imgs != null) {
            List<String> list = ListUtil.splitTrimExcludeEmpty(imgs, ',');
            if (list.size() > 0) {
                String oname = list.get(0);
                if (oname != null) {
                    if (oname.startsWith("/")) {
                        oname = oname.substring(1);
                        int idx = oname.indexOf('.');
                        if (idx > 0) {
                            String dname = oname.substring(0, idx) + "_s.jpg";
                            File dest = new File(this.imgPath, dname);
                            if (dest.exists())
                                return;
                            File src = new File(this.imgPath, oname);
                            if (src.exists()) {
                                byte[] obs = IoUtil.readStream(new FileInputStream(src), true);
                                obs = JpgUtil.read(obs);
                                FileOutputStream out = new FileOutputStream(dest);
                                try {
                                    JpgUtil.scalingZoom(new ByteArrayInputStream(obs), out, this.imgMaxWidth);
                                    out.flush();
                                } finally {
                                    IoUtil.close(out);
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private void remove(List<UploadFile> list) {
        for (UploadFile file : list) {
            if (file.fn != null)
                file.fn.delete();
        }
    }

    private UploadFile buildTargetFile(String ext) {
        StringBuilder sb = new StringBuilder();
        sb.append(System.currentTimeMillis() / (1000 * 60 * 60 * 24));
        File path = new File(this.imgPath, sb.toString());
        if (!path.exists()) {
            synchronized (this) {
                if (!path.mkdirs())
                    throw new RuntimeException("mkdir error[" + path + "]");
                FN_IDX.set(1);
            }
        }
        File file;
        do {
            String fn = FN_IDX.toString();
            if (ext.isEmpty()) {
                file = new File(path, fn);
            } else {
                file = new File(path, fn + "." + ext);
            }
            FN_IDX.incrementAndGet();
        } while (file.exists());
        sb.append("/").append(file.getName());
        UploadFile uf = new UploadFile();
        uf.setFn(file);
        uf.setUri("/" + sb.toString());
        return uf;
    }

    private String normalizeFileName(String fn) {
        int index = fn.indexOf('\\');
        if (index >= 0) {
            fn = fn.substring(fn.lastIndexOf('\\') + 1);
        }
        index = fn.indexOf('/');
        if (index >= 0) {
            fn = fn.substring(fn.lastIndexOf('/') + 1);
        }
        if (fn.length() == 0)
            throw new RuntimeException("invalid filename in Multipart/data request");
        return fn;
    }

    public static class UploadFile {
        private String name;
        private String uri;
        private long size;
        private transient File fn;

        public File getFn() {
            return fn;
        }

        public void setFn(File fn) {
            this.fn = fn;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getUri() {
            return uri;
        }

        public void setUri(String uri) {
            this.uri = uri;
        }

        public long getSize() {
            return size;
        }

        public void setSize(long size) {
            this.size = size;
        }
    }
}
