package com.ekexiu.portal.question;

import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.Batch;
import org.jfw.apt.orm.annotation.dao.Column;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.IncludeFixSet;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.SetSentence;
import org.jfw.apt.orm.annotation.dao.method.Where;
import org.jfw.apt.orm.annotation.dao.method.operator.DeleteWith;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.LimitQuery;
import org.jfw.apt.orm.annotation.dao.method.operator.LimitSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.QueryVal;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.In;
import org.jfw.apt.orm.annotation.dao.param.LessThan;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.apt.orm.annotation.dao.param.SqlColumn;
import org.jfw.apt.orm.core.defaultImpl.LongHandler;
import org.jfw.apt.orm.core.defaultImpl.StringHandler;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;


@DAO
public interface QuestionDao {

	@Insert
	int insert(Connection con, Question qen) throws SQLException;

	@DeleteWith
	@From(QetKeyWord.class)
	int deleteKeys(Connection con, String id) throws SQLException;
	
	
	@UpdateWith
	@From(Question.class)
	@IncludeFixSet
	@SetSentence("REPLY_COUNT=REPLY_COUNT+1")
	int incQuestionReply(Connection con,String id)throws SQLException;
	
	@UpdateWith
	@From(Question.class)
	@SetSentence("REPLY_COUNT=REPLY_COUNT+1")
	int onlyIncQuestionReply(Connection con,String id)throws SQLException;
	
	
	
	@UpdateWith
	@From(Question.class)
	@SetSentence("REPLY_COUNT=REPLY_COUNT-1")
	@Where("REPLY_COUNT >0")
	int decQuestionReply(Connection con,String id)throws SQLException;
	
	@UpdateWith
	@From(Question.class)
	@SetSentence("PAGE_VIEWS=PAGE_VIEWS+1")
	int incQuestionPageViews(Connection con,String id)throws SQLException;

	@Insert
	@Batch
	int[] insert(Connection con, QetKeyWord[] kws) throws SQLException;

	@LimitSelect
	@OrderBy("ORDER BY TIME_DESC DESC")
	List<Question> query(Connection con, @Nullable String state, @Nullable String uid, @LessThan String timeDesc, int rows) throws SQLException;

	@LimitSelect
	@OrderBy("ORDER BY TIME_DESC DESC")
	@Where(" STATE='1'")
	List<Question> watch(Connection con,
			@SqlColumn(handlerClass = StringHandler.class, value = {
					" ID IN (SELECT WATCH_OBJECT FROM WATCH WHERE WATCH_TYPE='8' AND PROFESSOR_ID=?)" }) String uid,
			@LessThan String timeDesc, int rows) throws SQLException;

	@SelectOne
	@Nullable
	Question query(Connection con, String id) throws SQLException;
	
	@UpdateWith
	@From(Answer.class)
	@SetSentence("STATE='0'")
	@Where("STATE ='1'")
	int logicDeleteAnswer(Connection con,String id,String qid)throws SQLException;
	
	@UpdateWith
	@From(Answer.class)
	@SetSentence("STATE='1'")
	@Where("STATE <>'1'")
	int unDeleteAnswer(Connection con,String id,String qid)throws SQLException;

	
	@SelectOne
	@Nullable
	Answer queryAnswer(Connection con,String id)throws SQLException;

	@SelectList
	List<Question> query(Connection con, @In String[] id) throws SQLException;
	
	@LimitSelect
	@OrderBy("ORDER BY TIME_DESC DESC")
	@Where("QID IN(SELECT ID FROM QUESTION WHERE STATE='1') AND STATE='1'")
	List<Answer> answer(Connection con,@LessThan String timeDesc, int rows) throws SQLException;

	@LimitSelect
	@OrderBy("ORDER BY TIME_DESC DESC")
	@Where("QID IN(SELECT ID FROM QUESTION WHERE STATE='1') AND STATE='1'")
	List<Answer> answerSelf(Connection con, String uid, @LessThan String timeDesc, int rows) throws SQLException;
	
	@SelectOne
	@Nullable
	Answer answer(Connection con,String uid,String qid)throws SQLException;

	@LimitSelect
	@OrderBy("ORDER BY TIME_DESC DESC")
	@Where("QID IN(SELECT ID FROM QUESTION WHERE STATE='1') AND STATE='1'")
	List<Answer> watchAnswer(Connection con,
			@SqlColumn(handlerClass = StringHandler.class, value = {
					" ID IN (SELECT WATCH_OBJECT FROM WATCH WHERE WATCH_TYPE='9' AND PROFESSOR_ID=?)" }) String uid,
			@LessThan String timeDesc, int rows) throws SQLException;

	@LimitSelect
	@Where("STATE='1'")
	@OrderBy("ORDER BY TIME_DESC DESC")
	List<Answer> byQes(Connection con, String qid, @LessThan String timeDesc, int rows) throws SQLException;
	@LimitQuery
	@OrderBy("ORDER BY SCORE_DESC DESC")
	@Where("STATE='1'")
	List<SortedAnswwer> byQesScore(Connection con, String qid, @SqlColumn(handlerClass = StringHandler.class, value = { "(case when ballot=0 then '00000'  else  TO_CHAR(10000 *AGREE / BALLOT,'00009') end || TO_CHAR(AGREE,'00000000000000000009') || ID) < ?  " })  String score, int rows) throws SQLException;

	@Insert
	int insert(Connection con,AnswerAgreeRec aar)throws SQLException;
	@UpdateWith
	@From(AnswerAgreeRec.class)
	@Where("FLAG<>'1'")
	@SetSentence("FLAG='1'")
	int agree(Connection con,String uid,String aid)throws SQLException;
	@UpdateWith
	@From(AnswerAgreeRec.class)
	@Where("FLAG='1'")
	@SetSentence("FLAG='0'")
	int unAgree(Connection con,String uid,String aid)throws SQLException;
	
	@UpdateWith
	@From(Answer.class)
	int update(Connection con,String id, @Set( " = AGREE +?" ) long agree,@Set("=BALLOT+?") long ballot)throws SQLException;
	
	
	@Insert
	int insert(Connection con, Answer answer) throws SQLException;

	@UpdateWith
	@From(Answer.class)
	@Where("STATE='1'")
	@SetSentence("MODIFY_TIME=TO_CHAR(NOW(),'YYYYMMDDHH24MISS')")
	int updateAnswer(Connection con, String id, @Set String cnt) throws SQLException;
	
	@Insert
	int insert(Connection con,QuestionInviteRec rec)throws SQLException;
	
	@SelectList
	List<QuestionInviteRec> queryInviteRec(Connection con,String qid,String uid,@In String[] pid)throws SQLException;
	
	@SelectOne
	@Nullable
	AnswerAgreeRec queryAswerAgreRec(Connection con,String uid,String aid)throws SQLException;
	
	@DeleteWith
	@From(AnswerAgreeRec.class)
	int deleteAnswerAgreeRec(Connection con,String uid,String aid,boolean flag)throws SQLException;
	
	@DefaultValue("0")
	@QueryVal
	@Column(value="COUNT(1)",handlerClass=LongHandler.class)
	@From(Answer.class)
	long answerCount(Connection con,@Nullable String uid,@Nullable String qid, @Nullable String state)throws SQLException;

	@DefaultValue("0")
	@QueryVal
	@Column(value = "COUNT(1)", handlerClass = LongHandler.class)
	@From(Answer.class)
	@Where("QID IN(SELECT ID FROM QUESTION WHERE STATE='1') AND STATE='1'")
	long answerSelfCount(Connection con, String uid)throws SQLException;
}
